
#include <sys/p2es_clock.h>
#include <inttypes.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>


#define MIN(a,b) (((a)<(b))?(a):(b))
#define MAX(a,b) (((a)>(b))?(a):(b))



__pasm {
ORG 0
startADC
		setq	#3
		rdlong	.launcher, ptra			' fetch the stored cog ID of launcher cog ... and the other three parameters
		rczr	.launcher+1		wcz	' extract Sinc# parameter
		shl	.launcher+1, #2			' restore pin mode
	if_nc	or	.launcher+1, #P_COUNT_HIGHS	' Sinc1
	if_c	or	.launcher+1, #P_ADC		' Sinc2 & 3

		fltl	#ADCPINS
		wrpin	.launcher+1, #ADCPINS		' turn on ADC modulators and input/gain, and select smartpin mode
	if_nc	wxpin	.launcher+2, #ADCPINS		' set Sinc1 decimation/sample period in sysclocks
	if_nc	wypin	#0, #ADCPINS
if_c_and_nz	wxpin	#%01_0000, #ADCPINS		' Sinc2 filter sub-mode
if_c_and_z	wxpin	#%10_0000, #ADCPINS		' Sinc3 filter sub-mode
	if_c	wypin	.launcher+2, #ADCPINS		' set decimation period in sysclocks
		dirh	#ADCPINS			' enable smartpins
		waitx	##_clkfreq/100			' 10 ms pause for ADC modulators to warm up

		wrfast	.launcher+3, ptra		' initiate FIFO with specified wrap length, PTRA is hubRAM address of "mailbox"
		rdpin	pa, #ADCPIN			' re-arm smartpin, make IN low
		setse1	#(%001<<6) | ADCPIN		' event trigger on rising edge of IN
		waitse1					' start synchronised
if_c_and_z	jmp	#.sinc3

.sinc1_2
	if_c	mov	.diff2+1, #.diff1
	if_c	mov	.diff2+2, ##.diff1 | (1<<9)
.loop1
		mov	pb, #ADCPIN
	if_c	mov	.diff2, #0

		rep	@.chrend, #8
		rdpin	pa, pb
	if_c	alts	.diff2, .diff2+1		' non-incrementing
	if_c	sub	pa, 0-0				' diff1..diff1+7
	if_c	altd	.diff2, .diff2+2		' incrementing
	if_c	add	0-0, pa				' diff1..diff1+7
	if_c	zerox	pa, #26
		wflong	pa
		add	pb, #1
.chrend
		waitse1					' wait for next decimation event
		jnfbw	#.loop1				' loop until FIFO wrapping event, don't want remaining FIFO data
		jmp	#.done

.sinc3
		rdpin	pa, #ADCPIN
		sub	pa, .diff1
		add	.diff1, pa
		sub	pa, .diff2
		add	.diff2, pa
		zerox	pa, #26
		wflong	pa

		rdpin	pa, #ADCPIN+1
		sub	pa, .diff1+1
		add	.diff1+1, pa
		sub	pa, .diff2+1
		add	.diff2+1, pa
		zerox	pa, #26
		wflong	pa

		rdpin	pa, #ADCPIN+2
		sub	pa, .diff1+2
		add	.diff1+2, pa
		sub	pa, .diff2+2
		add	.diff2+2, pa
		zerox	pa, #26
		wflong	pa

		rdpin	pa, #ADCPIN+3
		sub	pa, .diff1+3
		add	.diff1+3, pa
		sub	pa, .diff2+3
		add	.diff2+3, pa
		zerox	pa, #26
		wflong	pa

		rdpin	pa, #ADCPIN+4
		sub	pa, .diff1+4
		add	.diff1+4, pa
		sub	pa, .diff2+4
		add	.diff2+4, pa
		zerox	pa, #26
		wflong	pa

		rdpin	pa, #ADCPIN+5
		sub	pa, .diff1+5
		add	.diff1+5, pa
		sub	pa, .diff2+5
		add	.diff2+5, pa
		zerox	pa, #26
		wflong	pa

		rdpin	pa, #ADCPIN+6
		sub	pa, .diff1+6
		add	.diff1+6, pa
		sub	pa, .diff2+6
		add	.diff2+6, pa
		zerox	pa, #26
		wflong	pa

		rdpin	pa, #ADCPIN+7
		sub	pa, .diff1+7
		add	.diff1+7, pa
		sub	pa, .diff2+7
		add	.diff2+7, pa
		zerox	pa, #26
		wflong	pa

		waitse1					' wait for next decimation event
		jnfbw	#.sinc3				' loop until FIFO wrapping event, don't want remaining FIFO data

.done
		dirl	#ADCPINS
		wrpin	#0, #ADCPINS
		decod	.launcher
		cogatn	.launcher
		cogid	pa
		cogstop	pa


.launcher	long	0
.diff1		res	8
.diff2		res	8
}


__pasm {
ORG 0
startTog
		setq	#3
		rdlong	.params, ptra
		shr	.params+3, #2		' buffer length, measured in 64-byte blocks
		mov	pa, .params+2		' decimation/sample period
		mul	pa, .params+3
		sub	pa, #4
		dirh	#ADCPINS		' DIR masks the OUT in each cog separately

		rep	@.rend, #0		' loop forever
		waitx	pa
		outnot	#ADCPINS
.rend

.params		res	4
}


static void  emit_adc_mode( int32_t mode )
{
	printf( " Sinc order: %d\n ADC source/gain: ", mode&3 );
	mode = (mode >> 15) & 7;
	switch( mode )
	{
		case 0:
			puts( "P_ADC_GIO" );
			break;
		case 1:
			puts( "P_ADC_VIO" );
			break;
		case 2:
			puts( "P_ADC_FLOAT" );
			break;
		case 3:
			puts( "P_ADC_1X" );
			break;
		case 4:
			puts( "P_ADC_3X" );
			break;
		case 5:
			puts( "P_ADC_10X" );
			break;
		case 6:
			puts( "P_ADC_30X" );
			break;
		case 7:
			puts( "P_ADC_100X" );
			break;
		default:
			puts( "unknown" );
			break;
	}	
}


enum {
	_clkfreq = 250_000_000,

	ADCPIN = 40,
	ADCPINS = ADCPIN | (7<<6),
};



int  main( void )
{
	int32_t volatile * mailbox;
	int32_t  mb1;
	unsigned  idx, id;


	mailbox = __builtin_alloca( 5000 * 64 + 8 );		// 10_000 samples x 8 channels
	if( mailbox == NULL )  return( 1 );
	memset( mailbox, 0, 5000 * 64 + 8 );

	mailbox[0] = _cogid();					// launcher (self) cog ID for launchee COGATN
	mailbox[1] = mb1 = P_ADC_1X | P_OE | 1;			// ADC source/gain, and Sinc order
	mailbox[2] = 50000;					// decimation/sample period
	mailbox[3] = 5000;					// buffer length, measured in 64-byte blocks

	_waitms( 200 );						// pause for terminal ready
	printf( "\n\n\n\n\n\n\n\n\n\n Sysclock frequency: %d,%03d,%03d Hz\n", _clkfreq/1000000, (_clkfreq/1000)%1000, _clkfreq%1000 );
	printf( " Sampling frequency: %d Hz\n", _clkfreq / mailbox[2] );
	emit_adc_mode( mb1 );

	id = _coginit( COGEXEC_NEW, &startADC, mailbox );	// start another cog to record eight ADC channels via Sinc3 smartpin mode
	if( id == -1 )  return( 2 );

	mb1 &= P_OE;
	if( mb1 )
	{
		id = _coginit( COGEXEC_NEW, &startTog, mailbox );	// start another cog to bit-bash a square wave at the ADCs
		if( id == -1 )  return( 3 );
	}

	_waitatn();						// twiddle thumbs waiting for the other cog to complete
	if( mb1 )  _cogstop( id );

	idx = 80;						// start from 10 samples into the recorded data
	printf( "\n P%d P%d P%d P%d P%d P%d P%d P%d\n", ADCPIN, ADCPIN+1, ADCPIN+2, ADCPIN+3, ADCPIN+4, ADCPIN+5, ADCPIN+6, ADCPIN+7 );
	do {
		printf( " %d %d %d %d %d %d %d %d\n", mailbox[idx], mailbox[idx+1], mailbox[idx+2], mailbox[idx+3], mailbox[idx+4], mailbox[idx+5], mailbox[idx+6], mailbox[idx+7] );
		idx += 8;
	} while( idx < 80_000 );

	puts( " STOP" );
	_cogstop( _cogid() );

	return( 0 );
}
